C=======================================================================
C
C  FOURIER calculations for arbitrary space groups in crystallography
C     from input data in maximum-entropy program SMEED format.
C
C     Symmetry equivalent reflections to the ones given are calculated 
C     automatically: it is assumed, that powder reflections are given.
C
C  Author: K. Burger (Inst. for crystallography, Univ. Tuebingen, Germany) 
C          march, june, oct. 1995, april, mai, october 1996, March 1998
C  Copyright by the author. Use for non-profit purpose is free.
C
C
C  some routines have been taken from the maximum-entropy program SMEED:
C
C       MAXIMUM ENTROPY ELECTRON DENSITY ANALYSIS PROGRAM
C       FOR X-RAY POWDER DIFFRACTION DATA
C       REFERENCE : J.APPL.CRYST. (1990) 23, 526-534
C       COPYRIGHT BY SHINTARO KUMAZAWA ( NAGOYA UNIVERCITY )
C       ALL RIGHT RESERVED.
C
C  - UNIT NUMBER OF DATA FILES -
C
C     10 : MEED data file ( THIS IS POWLS6.EXE COMPATIBLE FORMAT. )
C     20 : density data file 
C     21 : xy-map file for calculation of a section
C     30 : document data file
C     40 : space group data base 'SPGRA', offered by Fujio Izumi.
C
C=======================================================================
C
C
      PROGRAM fourier
C
C     *** mref = max. number of reflections in single crystal case     
C     *** masym= max. number of pixels in asymmetric unit of the cell
C     *** mpix = max. pixel resolution along a crystal cell axis.
C     *** nasym= number of points in asymmetric unit
C     *** nref = number of (non-zero) reflections
C     *** IASYM= array with coordinates (in pixel units) of pixels of a.u.
C     *** NPIX(3)=array with number of pixels alog cell-edges
C     *** RDEN() =array with density at each pixel
C     *** F()  = array with real and imaginary part of structure factors
C     *** rdint/av = total charge (in electrons) in unit cell.  
C     *** av = cell volume of crystal unit cell
C
      IMPLICIT  none
      integer*4 masym,nasym
      integer mref,mpix
      PARAMETER (mref=5000,masym=27500,mpix=129)
      real*8 RDEN(masym)
      integer NPIX(3),IASYM(3,masym),iwhat
C
C     *** variables connected with the structure factors given as input:
C     *** F: unique reflections, GOBS: groups of non-unique refl.
      real*8 F(2,mref),SIGF(mref),PROF(mref),GOBS(mref),
     $   SIGG(mref),PMUL(mref),PMULS(mref)
      integer IDX(3,mref),NGRP(mref),nref,nref1,nref2,mult(mref)
C
C     *** auxilary array for mkasym subroutine
      integer IDE(0:mpix,0:mpix,0:mpix)
C
C     *** cell-constants and others:
      real*8 CELL(6),pop
      integer iptyp,jph,jpk,jpl
      real*8 pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub,nsg,nset
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** variables for mapfxxx subroutine, which calculates a section
      real*8 P1(3),P2(3),P3(3),maplx,maply
      integer imapor,imappx,imappy,nadd
C
C     *** other variables ***      
      CHARACTER filmem*50,filden*50,aspg*10,title*200,tit2*200
      real*8 lambda,wert
      integer nyes,iwas,ierr,igsw,iref,i,j
      logical CR,error
C
1000  FORMAT(a)
1001  FORMAT(a,'  [default=',f8.5,']')
1002  FORMAT(a,'  [default=',i4,']')
 2010 FORMAT(BN,A30)
 2020 FORMAT(BN,I5)
 3500 FORMAT(/ ' INPUT MEED DATA SET NAME.     = ')
C 3550 FORMAT(  ' Are you sure ?  YES(0)/NO(1)  = ')
C
C     *** initial values for variables given to subroutine MAPFXXX
      nadd=1
      imapor=0
      maplx=5.0
      maply=5.0
      imappx=30
      imappy=30
C
C**********************************************************
C*** INPUT FILENAMES AND INITIAL CONDITIONS. **************
C**********************************************************
C
  200 print *
      WRITE(*,*) ' FOURIER calculations from data of ',
     +            'MaxEnt input-files'
      write(*,*) ' K. Burger, Univ. Tuebingen, Germany,'
     $  ,' March a.d. 1998'
      WRITE(6,3500)
      READ(5,2010,ERR=200) filmem
C
C
C*** READ MEED DATA AND MAKE SOME CONSTANTS. ***
C
      igsw=1
      CALL READMEM(filmem,title,CELL,pop,iptyp,jph,jpk,jpl,
     $  nsg,nset,NPIX,mpix,totale,lambda,mref,nref,nref1,nref2,
     $  IDX,F,SIGF,GOBS,SIGG,NGRP,PMUL,PMULS,ierr,igsw)
      if (ierr.ne.0) goto 200
      tit2='FOURIER: '//title(1:180)
      title=tit2

      CALL RDSPG(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)

C     *** display of symmetry info ***
C     CALL DSPSYM(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
      CALL PRECAL(jph,jpk,jpl,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2,CELL,NPIX)
C
      if (nref2.ne.0) print *,
     $ '(Non-unique reflections given as groups will be neglected)'
      nref2=0
C
      print *
      iwhat=0
105   write(*,*) 'Now, should symmetry equivalent reflections be'
     $           //' calculated?'
      write(*,1000) 
     $    ' 0 = Yes, because powder reflections or merged single-'
     $    //'crystal reflections'
      write(*,1000) '     have been read'
      write(*,1000)
     $    ' 1 = No, because un-merged single crystal reflections '
     $    //' have been read'
      write(*,1002) ' Your choice ? ',iwhat
      CALL ABFRAGE(wert,i,CR,error)
      if (error) goto 105
      if (.not.CR) then
         if (i.gt.1 .or. i.lt.0) then
           print *,'wrong input'
           goto 105
        else
          iwhat=i
        endif
      endif
C 
C     *** remove reflections with |F|=0, and if iwhat=0, then make a
C     *** complete set of reflections using the symmetry information
      CALL CALCMUL(iwhat,mref,nref,nref1,F,IDX,MULT,
     $  IROT,nsym,ncen,RTRS,pi) 
C
C      open (20,file='l3',STATUS='UNKNOWN')
C      do 110 iref=1,nref
C        write(20,*) (IDX(j,iref),j=1,3),(F(j,iref),j=1,2)
C110   continue
C      close(20)
C
C     *** calculate preferred orientation correction factors
      CALL POCOR(iptyp,pop,PROF,nref,mref,IDX,
     $	 as,bs,cs,ga,gb,gg,rh,rk,rl,pi)
C
      nyes=0
      print *
22    write(*,*) '     Do you like the'
      write(*,*) ' (0) observed  or the'
      write(*,*) ' (1) calculated structure factors ?'
      write(*,1002) ' Your choice ? ',nyes
      CALL ABFRAGE(wert,i,CR,error)
      if (error) goto 22
      if (.not.CR) then
        if (i.gt.1 .or. i.lt.0) then
          print *,'wrong input'
          goto 22
        else
          nyes=i
        endif
      endif
      if (nyes.eq.1) CALL READDOC(iptyp,mref,nref,nref1,F,IDX)
C
C     ***  What is to do ? ***
C
33    write(*,*) '    Do you like the calculation of ' 
      write(*,*) ' (0) the asymmetric unit'
      write(*,*) ' (1) an arbitrary plane ?'
      iwas=0
      write(*,1002) ' Your choice ? ',iwas
      CALL ABFRAGE(wert,i,CR,error)
      if (error) goto 22
      if (.not.CR) then
        if (i.gt.1 .or. i.lt.0) then
          print *,'wrong input'
          goto 33
        else
          iwas=i
        endif
      endif
      if (nyes.eq.1) CALL READDOC(iptyp,mref,nref,nref1,F,IDX)
      
      if (iwas.eq.0) then
        print *,'Please input the name of a density file with the '
	print *,'same asymm. unit: (or RETURN)'
35      read(5,2010,ERR=35) filden
	if (filden(1:10).eq.'          ') then
C         *** make asymmetric unit from space group information ***
          CALL MKASY2(masym,nasym,mpix,NPIX,
     $      IROT,RTRS,RSUB,nsym,ncen,nsub,RDEN,rdint,IASYM,IDE)
	  write(*,'(I7,a40)') nasym, 
     $      ' points in the asymmetric unit.'
        else
C         *** read asym. unit from existing example file ***
          CALL READDEN(filden,masym,nasym,IASYM,RDEN)
        endif

        CALL FCALC1(mref,nref,IDX,F,
     $    masym,nasym,NPIX,IASYM,RDEN,rdint,av,pi)

        print *
        write(*,*) 'Filename for output of asymm. unit density ?'
        read(*,'(a50)') filden
        CALL SAVEDEN(filden,
     $    masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $    ,nsym,ncen,nsub,IROT,RTRS,RSUB)
      ELSE    
C       *** arbitrary plane to calculate ***
        CALL MAPFXXX(mref,nref,IDX,F,rdint,av,nadd,
     $    mpix,NPIX,filmem,title,
     $    CELL,ca,cb,cg,sa,sb,sg,as,bs,cs,ga,gb,gg,pi,
     $    P1,P2,P3,imapor,maplx,maply,imappx,imappy)
      ENDIF
C
      write(*,*) 'Finished.'
      print *
      write(*,*) 'Something else to do? (1=yes)'
      nyes=0
      read(*,2020) nyes
      if (nyes.eq.1) goto 22

      STOP
      END

C*** END OF PROGRAM ***
