C--------------------------------------------------------------------
C     MAKES DATA FOR THE 2D MAP OF THE (110)-PLANE
C--------------------------------------------------------------------
C
      SUBROUTINE MAP110(mapt,mpix,NPIX,nmx,nmy,vx,vy,theta,rad,
     $  CELL,DELC,D,lx,ly,dmin,dmax,axisX,axisY,
     $  imap11,imapso,imapsx,imapsy,imapuc,imaptr)
C
      IMPLICIT none
      integer mpix,mapt,imap11,imapso,imapsx,imapsy,imapuc,imaptr
C
C     *** mpix=max. number of pixels along cell-edges 
C     *** mapt=type of map: 1: (110), 2: (100), 3: (010), 4: (001)
C     ***      which is passed to the suboutine tr2d which transposes
C     ***      the 2-dim map axes: thus the axes' names are known there
C     *** imap11=which kind of diagonal: 110 of -110
C     *** imapsf,-sx,-sy,-uc,-tr: input from subroutines
C
C     *** DELC(0..npix(1)-1, 0.. , 0..) contains the whole cell 3-dim density
      real*8 DELC(0:mpix,0:mpix,0:mpix)
C     *** D(1..lx,1..ly) gets the 2-dim map from this subroutine
      real*8 D(2*mpix+1,2*mpix+1)
C     *** CELL() contains the cell-constants
      real*8 CELL(6)
C     *** NPIX() contains the number of pixels along the three cell-axes
C     *** nmx,nmy,lx,ly refer to the 2-dim map, and lx,ly may be 2*nx+1,2*ny+1
C     ***               if the map is doubled (by subroutine ucmul)
      integer NPIX(3),nmx,nmy,lx,ly
      integer i,j,nh,nv,ndmv
C     *** vx,vy contain the cell-axes' lengths in Angstrom for the 2-dim map 
C     *** theta contains the angle between the two edges
      real*8 a3,a4,a12,rad,cs1,cs2,vx,vy,theta,dmin,dmax,dd,wert
      logical CR,error
      CHARACTER axisX*5,axisY*5
C
C 2000 FORMAT(BN,I3)
 3010 FORMAT(/
     $  ' Select the type of plane. 0: (-110), 1: (110) ',
     $  ' [default=',i2,'] ',$)
C
C** SETS & CALCULATES GRAPHIC PARAMETERS ***
C
      nmx=NPIX(1)
      nmy=NPIX(3)
10    WRITE(*,3010) imap11
      CALL abfrage(wert,i,CR,error)
      if (error) goto 10
      if (.not.CR) then
         if (i.gt.1 .or. i.lt.0) then
           print *,'wrong input: has to be 0 or 1'
           goto 10
        else 
          imap11=i
        endif
      endif
C
      if (imap11.eq.0) then
        print *,'(-110) diagonal.'
        print *,'horizontal axis= diagonal from O[0,0,0] to [1,1,0]'
        print *,'vertical axis  = c-axis  '
      else
        print *,'(110) diagonal.'
        print *,'horizontal axis= diagonal from [0,1,0] to [1,0,0]'
        print *,'vertical axis  = c-axis,   origin on [0,1,0]'
      endif
      print *
C     *** a3=2*a*b*cos(gamma)
      A3=2*CELL(1)*CELL(2)*DCOS(rad*CELL(6))
C     *** a4=2*a*b*cos(180-gamma)=-2*a*b*cos(gamma)
      A4=2*CELL(1)*CELL(2)*DCOS(rad*(180.0-CELL(6)))
      IF (imap11.NE.0) THEN
C        *** exchange a3,a4
         A12=A3
         A3=A4
         A4=A12
      END IF
C     a12=a^2 + b^2
      A12=CELL(1)*CELL(1)+CELL(2)*CELL(2)
      CS1=DSQRT(A12+A3)
      CS2=(CELL(1)*DCOS(rad*CELL(5))+CELL(2)*DCOS(rad*CELL(4)))/CS1
C     *** vx=(length of diagonale)  
      vx=DSQRT(A12-A4)
C     *** vy=(length of y-axis of 2-dim map) = c  
      vy=CELL(3)
      theta=DACOS(CS2)
C
C*** CONVERTS 3D DATA INTO 2D DATA, DELC --> D ***
C
C     imap11 = 0/1 for (-110)/(110) map
C
      DO 20 NH=1,nmx
C       *** x-axis
        IF (imap11.EQ.0) THEN
           NDMV=NH-1
        ELSE
           NDMV=nmx-NH+1
        END IF
        DO 30 NV=1,nmy
C         *** y-axis
          IF (imap11.EQ.1.AND.NH.EQ.1) THEN
             DD=DELC(0,0,NV-1)
          ELSE
             DD=DELC(NH-1,NDMV,NV-1)
          END IF
          D(NH,NV)=DD
   30   CONTINUE
   20 CONTINUE
C
C*** calculate maximum and minimum values ***
C
      dmin=d(1,1)
      dmax=d(1,1)
      do 22 i=1,nmx
      do 22 j=1,nmy
	 dd=d(i,j)
         IF (DD.LT.DMIN) DMIN=DD
         IF (DD.GT.DMAX) DMAX=DD
22    continue
C
C*** EDITS 2D DATA ***
C
C     *** Shift the origin ?
      CALL SFTDAT(mpix,nmx,nmy,D,imapso,imapsx,imapsy)
C     *** Transpose axes (x<-->y) ?
      CALL TR2D(mapt,mpix,nmx,nmy,D,vx,vy,axisX,axisY,imaptr)
C     *** Doubling of the map ?
      CALL UCMUL(mpix,nmx,nmy,lx,ly,D,imapuc)
C     *** Add values for two edges
      CALL EDGED(mpix,lx,ly,D)
C
      RETURN
      END
C
