C--------------------------------------------------------------------
C     READ SPACE GROUP DATA BASE AND MAKE SYMMETRY OPERATIONS.
C--------------------------------------------------------------------
C
      SUBROUTINE RDSPG(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)
C
C     *** nsg = number of space group (1<=nsg<=230)
C     *** nset = number of s.g. setting (1<=nset<=2)
C     *** ncen = 0/1 for no/a center of inversion present in the cell
C     *** nsym = number of symetry operators (1<=nsym<=48)
C     *** nsub = number of sub-translations in the cell because of 
C     *** IROT() = array of all symmetry operators' rotation matrices
C     *** RTRS() = array of corresponding translation vectors
C     *** RSUB() = transl. vectors belonging to cell-sub-translations
C     *** nsg: Number of space group, nset: its setting (1 or 2)
C     *** aspg: Name of space group
C     *** nsym: Number of symmetry operators 
C     *** ncen: 0/1 for non-existence/existence of a center of inversion
C     *** nsub: Number of cell sub-translatins (1..4)
C
      IMPLICIT none
      integer IROT(3,3,48),nsg,nset,nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4),CELL(6)
      CHARACTER ASPG*10,COORD(48)*20,CH*1,XYZ(3)*1,SYM(3)*10,
     $          CD*20,SYMM*10,filsym*40
      integer ixx,isym,i,j,i1,i2,isg,iset,idummi,nn,
     4        it,itr1,itr2
      logical yesno,yesno2
      DATA (XYZ(J),J=1,3)/'X','Y','Z'/
C
 1000 FORMAT(5I3,3X,A10)
 1010 FORMAT(14I2)
 1020 FORMAT(A20)
 1030 FORMAT()
C
C*** READ DATA BASE ***
C
C*** CAUTION : you must replace the file-name by your own one ***
      INQUIRE(FILE='SPGRA.DAT',EXIST=yesno)
      if (yesno) then
        filsym='SPGRA.DAT'
      else
        INQUIRE(FILE='spgra.dat',EXIST=yesno)
        if (yesno) filsym='spgra.dat'
      endif
      if (.not.yesno) then  
        INQUIRE(FILE='/home/burger/maxent/SPGRA.DAT',EXIST=yesno2) 
        if (yesno2) then
          filsym='/home/burger/maxent/SPGRA.DAT'
        else
          CALL PROEND(
     $    'File SPGRA.DAT with symmetry information not found.')
        endif
      endif
      OPEN(40,FILE=filsym,FORM='FORMATTED',STATUS='OLD',
     $     ACCESS='SEQUENTIAL',ERR=999)
      print *
      print *,'Symmetry information will be read from file ',filsym
      GOTO 10
C     ----------------
C     *** error ***
999   print *,'File ',filsym
      print *,'was found but could not be opened! '
      CALL PROEND(
     $  'File SPGRA.DAT with symmetry information unreadable.')
C
C     ----------------
   10 READ(40,1000,END=20) ISG,ISET,idummi,NCEN,NSYM,ASPG
      IF (NSG.EQ.ISG.AND.NSET.EQ.ISET) THEN
         READ(40,1010) (idummi,I=1,6),NN,(idummi,I=8,14)
         READ(40,1020) (COORD(I),I=1,NSYM)
         GO TO 25
      ELSE
         DO 15 idummi=1,NSYM+1
            READ(40,1030)
   15    CONTINUE
      END IF
      GO TO 10
   20 CALL PROEND('CAN NOT FIND YOUR SPACE GROUP SELECTION.')
C
C*** CONVERT STRING INTO ROT MATRIX AND TRS VECTOR. ***
C
   25 DO 30 isym=1,NSYM
C        *** loop over all symmetry operators ***
         cd=COORD(isym)
C        *** find position of the two kommas ','
         I1=INDEX(cd,',')
         DO 40 j=i1+2,19
            READ(CD(j:j),'(A1)') ch
            IF (ch.EQ.',') THEN
               i2=j
               GO TO 50
            END IF
   40    CONTINUE
   50    SYM(1)=CD(1:i1-1)
         SYM(2)=CD(i1+1:i2-1)
         SYM(3)=CD(i2+1:20)
         DO 80 i=1,3
C           *** loop over three SYM()-strings given for one operator
C           *** for the x,y,z-row of the matrix and transl.-vector
            symm=SYM(i)
            RTRS(i,isym)=0
            DO 60 j=1,3
               IROT(i,j,isym)=0
   60       CONTINUE
C           *** extract rotation matrix
            DO 70 j=1,3
c              *** loop over three possible letters x,y,z ***
               ixx=INDEX(symm,XYZ(j))
               IF (ixx.GT.0) THEN
                  IF (ixx.EQ.1) THEN
                     IROT(i,j,isym)=1
                  ELSE
                     IF (symm(ixx-1:ixx-1).EQ.'+') THEN
                        IROT(i,j,isym)=1
                     ELSE
                        IROT(i,j,isym)=-1
                     END IF
                  END IF
               END IF
   70       CONTINUE
C           *** extract translation vector
            it=INDEX(symm,'/')
            IF (it.GT.0) THEN
               READ(symm(it-1:it-1),'(I1)') itr1
               READ(symm(it+1:it+1),'(I1)') itr2
               IF (it.NE.2) then
                 if (symm(it-2:it-2).EQ.'-') itr1=-itr1
               endif
               IF (itr2.NE.0) RTRS(I,isym)=itr1/(1.0*itr2)
            END IF
   80    CONTINUE
   30 CONTINUE
C
C*** MAKE SUB-LATTICE TRS VECTORS. ***
C
      NSUB=1
      IF (NN.EQ.7) THEN
C*** F LATTICE ***
         RSUB(2,2)=0.5
         RSUB(3,2)=0.5
         RSUB(1,3)=0.5
         RSUB(3,3)=0.5
         RSUB(1,4)=0.5
         RSUB(2,4)=0.5
         NSUB=4
      ELSE IF (NN.EQ.4) THEN
C*** A LATTICE ***
         RSUB(2,2)=0.5
         RSUB(3,2)=0.5
         NSUB=2
      ELSE IF (NN.EQ.5) THEN
C*** B LATTICE ***
         RSUB(1,2)=0.5
         RSUB(3,2)=0.5
         NSUB=2
      ELSE IF (NN.EQ.6) THEN
C*** C LATTICE ***
         RSUB(2,2)=0.5
         RSUB(1,2)=0.5
         NSUB=2
      ELSE IF (NN.EQ.13) THEN
C*** I LATTICE ***
         RSUB(1,2)=0.5
         RSUB(2,2)=0.5
         RSUB(3,2)=0.5
         NSUB=2
      ELSE IF (NN.EQ.14) THEN
C*** RHOMBO HEDRALLY CENTERED ( OBVERSE SETTING ) ***
         RSUB(1,2)=2.0/3
         RSUB(2,2)=1.0/3
         RSUB(3,2)=1.0/3
         RSUB(1,3)=1.0/3
         RSUB(2,3)=2.0/3
         RSUB(3,3)=2.0/3
         NSUB=3
      ELSE IF (NN.EQ.15) THEN
C*** RHOMBO HEDRALLY CENTERED ( REVERSE SETTING ) ***
         RSUB(1,2)=1.0/3
         RSUB(2,2)=2.0/3
         RSUB(3,2)=1.0/3
         RSUB(1,3)=2.0/3
         RSUB(2,3)=1.0/3
         RSUB(3,3)=2.0/3
         NSUB=3
      END IF
C
C     *** display of symmetry info ***
C     CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
      RETURN
      END
